@@ -4,7 +4,7 @@
  *              to build a moderately scalable and highly available server
  *              based on a cluster of servers, with more flexibility.
  *
- * Version:     $Id: tcp_vs.c,v 1.9 2002/05/11 08:23:38 wensong Exp $
+ * Version:     $Id: tcp_vs.c,v 1.11 2003/02/14 09:21:32 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -47,6 +47,7 @@ EXPORT_SYMBOL(register_tcp_vs_scheduler);
 EXPORT_SYMBOL(unregister_tcp_vs_scheduler);
 EXPORT_SYMBOL(tcp_vs_connect2dest);
 EXPORT_SYMBOL(tcp_vs_sendbuffer);
+EXPORT_SYMBOL(tcp_vs_xmit);
 EXPORT_SYMBOL(tcp_vs_recvbuffer);
 /*  EXPORT_SYMBOL(tcp_vs_lookup_dest); */
 EXPORT_SYMBOL(tcp_vs_getword);
@@ -55,6 +56,7 @@ EXPORT_SYMBOL(tcp_vs_exp_exec);
 #ifdef CONFIG_TCP_VS_DEBUG
 EXPORT_SYMBOL(tcp_vs_get_debug_level);
 #endif
+EXPORT_SYMBOL(sysctl_ktcpvs_unload);
 
 
 struct tcp_vs_conn *
@@ -113,12 +115,15 @@ skb_send_datagram_socket(const struct sk_buff *skb, struct socket *to)
 {
 	int written, i;
 	struct sk_buff *list;
+	int res;
 
 	if (!skb_is_nonlinear(skb))
 		return tcp_vs_sendbuffer(to, skb->data, skb->len, 0);
 
-	written = tcp_vs_sendbuffer(to, skb->data, skb->len - skb->data_len,
-				    MSG_MORE);
+	res = tcp_vs_sendbuffer(to, skb->data, skb_headlen(skb), MSG_MORE);
+	if (res < 0)
+		return res;
+	written = res;
 
 	for (i=0; i<skb_shinfo(skb)->nr_frags; i++) {
 		char *vaddr;
@@ -126,14 +131,21 @@ skb_send_datagram_socket(const struct sk_buff *skb, struct socket *to)
 		struct page *page = frag->page;
 
 		vaddr = kmap(page);
-		written += tcp_vs_sendbuffer
-			(to, vaddr + frag->page_offset, frag->size, 0);
+		res = tcp_vs_sendbuffer(to, vaddr + frag->page_offset,
+					frag->size, 0);
+		if (res < 0)
+			goto out;
+		written += res;
 		kunmap(page);
 	}
 
 	for (list = skb_shinfo(skb)->frag_list; list; list=list->next) {
-		written += skb_send_datagram_socket(list, to);
+		res = skb_send_datagram_socket(list, to);
+		if (res < 0)
+			goto out;
+		written += res;
 	}
+  out:
 	return written;
 }
 
@@ -179,16 +191,45 @@ static int tcp_vs_relay_socket(struct socket *from, struct socket *to)
 	}
 
 	if (!sysctl_ktcpvs_zerocopy_send) {
+		int res;
 		int written;
 
-		written = skb_send_datagram_socket(skb, to);
-		if (written != len)
-			TCP_VS_ERR_RL("relay socket data error (len=%d, "
-				      "written=%d).\n", len, written);
+		res = skb_send_datagram_socket(skb, to);
+		if (res < 0)
+			return res;
+		written = res;
+		if (written != len) {
+			if (skb_is_nonlinear(skb) &&
+			    skb_linearize(skb, GFP_ATOMIC) != 0) {
+				TCP_VS_ERR_RL("relay socket data error "
+					      "(len=%d, written=%d).\n",
+					      len, written);
+				kfree_skb(skb);
+				return written;
+			}
+		  sendagain:
+			res = tcp_vs_sendbuffer(to, skb->data+written,
+						skb->len-written, 0);
+			if (res < 0) {
+				kfree_skb(skb);
+				return written;
+			}
+
+			written += res;
+			if (written != len)
+				goto sendagain;
+		}
 		kfree_skb(skb);
 		return written;
 	}
 
+	/* we cannot release the skb here, but we do need to call
+	   its destructor so that the sock_rfree can update the
+	   source sk->rmem_alloc correctly. */
+	if(skb->destructor) {
+		skb->destructor(skb);
+	}
+
 	/* drop old route */
 	dst_release(skb->dst);
 	skb->dst = NULL;
@@ -201,9 +242,8 @@ static int tcp_vs_relay_socket(struct socket *from, struct socket *to)
 
 	lock_sock(sk);
 
-	TCP_VS_DBG(12, "forward_alloc=%d, rmem_alloc=%d\n",
-		   sk->forward_alloc, atomic_read(&sk->rmem_alloc));
-
+	/* set the owner of skb to the dest sk */
+	skb_set_owner_w(skb, sk);
 	skb_entail(sk, tp, skb);
 
 	skb->ip_summed = CHECKSUM_HW;
@@ -231,27 +271,43 @@ int tcp_vs_conn_handle(struct tcp_vs_conn *conn, struct tcp_vs_service *svc)
 	EnterFunction(5);
 
 	csock = conn->csock;
-	if ((csock->sk->state != TCP_ESTABLISHED) &&
-	    (csock->sk->state != TCP_CLOSE_WAIT) ) {
-		TCP_VS_ERR("Error connection not established\n");
+	if (csock->sk->state != TCP_ESTABLISHED) {
+		if (csock->sk->state == TCP_CLOSE_WAIT)
+			return 0;
+		TCP_VS_ERR("Error connection not established (state %d)\n",
+			   csock->sk->state);
 		return -1;
 	}
 
-	/* call its scheduler to select a server and create a socket
-	   to the destination server */
-	dsock = svc->scheduler->schedule(conn, svc);
-	if (!dsock) {
-		/* try to redirect the connection to other sockets */
+	/*
+	   call its scheduler to process the connect request, the scheduler may:
+	     1, Select a destination server and return 0, or;
+	     2, Deal with the requestion alone and return 1, or;
+	     3, Return -1  when could not find a right server
+	     4, return -2 when other errors such as socket broken occur.
+	*/
+	switch (svc->scheduler->schedule(conn, svc)) {
+	case 1:         /* scheduler has done all the work */
+		return 0;
+
+	case 0:         /* further process needed */
+		break;
+	case -1:	/* try to redirect the connection to other sockets */
 		if (svc->conf.redirect_port) {
 			redirect_to_local(conn, svc->conf.redirect_addr,
-					  svc->conf.redirect_port);
+			svc->conf.redirect_port);
 			return 0;
 		}
-
-		TCP_VS_ERR_RL("no destination available\n");
+		TCP_VS_ERR("no destination available\n");
+		return -1;
+	default:
 		return -1;
 	}
 
+	dsock = conn->dsock;
+	if (dsock == NULL)
+		return 0;
+
 	/*
 	 *  NOTE: we should add a mechanism to provide higher degree of
 	 *        fault-tolerance here in the future, if the destination
@@ -264,12 +320,12 @@ int tcp_vs_conn_handle(struct tcp_vs_conn *conn, struct tcp_vs_service *svc)
 
 	while (1) {
 		/* if the connection is closed, go out of this loop */
-		if (dsock->sk->state!=TCP_ESTABLISHED
-		    && dsock->sk->state!=TCP_CLOSE_WAIT)
+		if (dsock->sk->state != TCP_ESTABLISHED
+		    && dsock->sk->state != TCP_CLOSE_WAIT)
 			break;
 
-		if (csock->sk->state!=TCP_ESTABLISHED
-		    && csock->sk->state!=TCP_CLOSE_WAIT)
+		if (csock->sk->state != TCP_ESTABLISHED
+		    && csock->sk->state != TCP_CLOSE_WAIT)
 			break;
 
 		/* Do we have data from server? */
@@ -343,7 +399,8 @@ make_child(struct tcp_vs_child_table *tbl,
 		tbl->max_daemons_limit = slot + 1;
 	tbl->last_modified = jiffies;
 	tbl->children[slot].svc = svc;
-	if (kernel_thread(tcp_vs_child, &tbl->children[slot], 0) < 0)
+	if (kernel_thread(tcp_vs_child, &tbl->children[slot],
+			  CLONE_VM | CLONE_FS | CLONE_FILES) < 0)
 		TCP_VS_ERR("spawn child failed\n");
 }
 
@@ -385,7 +442,7 @@ child_pool_maintenance(struct tcp_vs_child_table *tbl,
 			}
 			break;
 		case SERVER_STARTING:
-		case SERVER_BUSY:
+			idle_count++;
 			last_non_dead = i;
 			break;
 		case SERVER_READY:
@@ -393,6 +450,9 @@ child_pool_maintenance(struct tcp_vs_child_table *tbl,
 			to_kill = i;
 			last_non_dead = i;
 			break;
+		case SERVER_BUSY:
+			last_non_dead = i;
+			break;
 		}
 	}
 	tbl->max_daemons_limit = last_non_dead + 1;
@@ -437,11 +497,12 @@ static int tcp_vs_child(void *__child)
 	struct socket *sock;
 	int ret = 0;
 	char *Buffer;
-	size_t BufLen=4096;
+	size_t BufLen;
 	struct tcp_vs_child *chd = (struct tcp_vs_child *)__child;
 	struct tcp_vs_service *svc = chd->svc;
 
 	DECLARE_WAIT_QUEUE_HEAD(queue);
+	/* DECLARE_WAITQUEUE(wait, current); */
 
 	EnterFunction(3);
 
@@ -449,7 +510,8 @@ static int tcp_vs_child(void *__child)
 	chd->pid = current->pid;
 	update_child_status(chd, SERVER_STARTING);
 
-	sprintf(current->comm,"ktcpvs c %s", svc->ident.name);
+	snprintf(current->comm, sizeof(current->comm),
+		 "ktcpvs %s c", svc->ident.name);
 	lock_kernel();
 	daemonize();
 
@@ -471,6 +533,7 @@ static int tcp_vs_child(void *__child)
 		ret = -ENOMEM;
 		goto out;
 	}
+	BufLen = PAGE_SIZE;
 
 	while (svc->stop == 0 && sysctl_ktcpvs_unload == 0) {
 		if (signal_pending(current)) {
@@ -479,12 +542,18 @@ static int tcp_vs_child(void *__child)
 			break;
 		}
 
+		update_child_status(chd, SERVER_READY);
 		if (sock->sk->tp_pinfo.af_tcp.accept_queue == NULL) {
-			update_child_status(chd, SERVER_READY);
+#if 0000
+			__set_current_state(TASK_INTERRUPTIBLE);
+			schedule();
+			__set_current_state(TASK_RUNNING);
+#endif
+			/* add_wait_queue(sock->sk->sleep, &wait); */
 			interruptible_sleep_on_timeout(&queue, HZ);
+			/* remove_wait_queue(sock->sk->sleep, &wait); */
 			continue;
 		}
-		update_child_status(chd, SERVER_BUSY);
 
 		/* create tcp_vs_conn object */
 		conn = tcp_vs_conn_create(sock, Buffer, BufLen);
@@ -492,13 +561,13 @@ static int tcp_vs_child(void *__child)
 			break;
 
 		/* Do the actual accept */
-		ret = sock->ops->accept(sock, conn->csock, 0);
+		ret = sock->ops->accept(sock, conn->csock, O_NONBLOCK);
 		if (ret < 0) {
-			TCP_VS_ERR("Error accepting socket\n") ;
 			tcp_vs_conn_release(conn);
-			break;
+			continue;
 		}
 
+		update_child_status(chd, SERVER_BUSY);
 		atomic_inc(&svc->conns);
 
 		/* Do the work */
@@ -529,13 +598,14 @@ static int tcp_vs_daemon(void *__svc)
 	int waitpid_result;
 	int i;
 	struct tcp_vs_service *svc = (struct tcp_vs_service *)__svc;
-	struct tcp_vs_child_table *child_table;
+	struct tcp_vs_child_table *child_table = NULL;
 
 	DECLARE_WAIT_QUEUE_HEAD(WQ);
 
 	atomic_inc(&tcp_vs_daemon_count);
 
-	sprintf(current->comm,"ktcpvs d %s", svc->ident.name);
+	snprintf(current->comm, sizeof(current->comm),
+		 "ktcpvs %s d", svc->ident.name);
 	lock_kernel();
 	daemonize();
 
@@ -571,7 +641,6 @@ static int tcp_vs_daemon(void *__svc)
 	/* Then wait for deactivation */
 	while (svc->stop == 0 && !signal_pending(current)
 	       && sysctl_ktcpvs_unload == 0) {
-		current->state = TASK_INTERRUPTIBLE;
 		interruptible_sleep_on_timeout(&WQ, HZ);
 
 		/* dynamically keep enough thread to handle load */
@@ -587,13 +656,15 @@ static int tcp_vs_daemon(void *__svc)
 
 	/* reap the zombie-daemons */
 	waitpid_result = 1;
-	while (waitpid_result>0)
+	while (waitpid_result > 0)
 		waitpid_result = waitpid(-1, NULL, __WCLONE|WNOHANG);
 
 	/* stop listening */
 	StopListening(svc);
 
- out:
+  out:
+	if (child_table)
+		vfree(child_table);
 	svc->start = 0;
 	atomic_set(&svc->running, 0);
 	atomic_dec(&tcp_vs_daemon_count);
@@ -614,8 +685,8 @@ static int master_daemon(void *unused)
 	MOD_INC_USE_COUNT;
 
 	sprintf(current->comm,"ktcpvs master");
-	lock_kernel();   /* This seems to be required for exit_mm */
-	exit_mm(current);
+	lock_kernel();
+	daemonize();
 
 	/* Block all signals except SIGKILL and SIGSTOP */
 	spin_lock_irq(&current->sigmask_lock);
@@ -655,7 +726,7 @@ static int master_daemon(void *unused)
 	/* flush all the virtual servers */
 	tcp_vs_flush();
 
-	TCP_VS_INFO("The master daemon stopped. \n"
+	TCP_VS_INFO("The master daemon stopped. "
 		    "You can unload the module now.\n");
 
 	MOD_DEC_USE_COUNT;
