@@ -11,7 +11,7 @@
 
 #include <asm/types.h>          /* For __uXX types */
 
-#define TCP_VS_VERSION_CODE		0x000008
+#define TCP_VS_VERSION_CODE		0x00000A
 #define NVERSION(version)                       \
 	(version >> 16) & 0xFF,                 \
 	(version >> 8) & 0xFF,                  \
@@ -53,6 +53,12 @@
 
 #define TCP_VS_TEMPLATE_TIMEOUT 15*HZ
 
+/* constants for http parsing */
+#define HTTP_MAX_REQLINE_SIZE		512  /* # bytes of a request line (pool)    */
+#define HTTP_MAX_STALINE_SIZE		512  /* # bytes of a status line (pool)     */
+#define HTTP_MAX_MIMLINE_SIZE		512  /* # bytes of a mime line (pool)       */
+
+
 struct tcp_vs_ident {
 	char		name[KTCPVS_IDENTNAME_MAXLEN];
 };
@@ -105,10 +111,10 @@ struct tcp_vs_rule_u {
 /* The argument to TCP_VS_SO_GET_INFO */
 struct tcp_vs_getinfo {
 	/* version number */
-	unsigned int	version;
+	unsigned int		version;
 
 	/* number of virtual services */
-	unsigned int	num_services;
+	unsigned int		num_services;
 };
 
 /* The argument to TCP_VS_SO_GET_SERVICE */
@@ -120,10 +126,10 @@ struct tcp_vs_service_u {
 	struct tcp_vs_config	conf;
 
 	/* number of real servers */
-	unsigned int	num_dests;
+	unsigned int		num_dests;
 
 	/* number of rules */
-	unsigned int	num_rules;
+	unsigned int		num_rules;
 
 	/* run-time variables */
 	unsigned int		conns;         /* connection counter */
@@ -214,10 +220,21 @@ extern int tcp_vs_get_debug_level(void);
 #define LeaveFunction(level)   do {} while (0)
 #endif
 
+/* switch off assertions (if not already off) */
+#ifdef CONFIG_TCP_VS_DEBUG
+#define assert(expr)						\
+	if(!(expr)) {						\
+		printk( "Assertion failed! %s,%s,%s,line=%d\n",	\
+		#expr,__FILE__,__FUNCTION__,__LINE__);		\
+	}
+#else
+#define assert(expr) do {} while (0)
+#endif
+
 
 #define KTCPVS_CHILD_HARD_LIMIT		512
 
-#define NET_KTCPVS              20
+#define NET_KTCPVS			20
 
 enum {
 	NET_KTCPVS_DEBUGLEVEL		= 1,
@@ -226,10 +243,6 @@ enum {
 	NET_KTCPVS_ZEROCOPY_SEND	= 4,
 };
 
-enum {
-	NET_KTCPVS_DUMMYVS         = 1,
-};
-
 
 /* very very simple expression currently */
 struct tcp_vs_expression {
@@ -308,6 +321,22 @@ struct tcp_vs_dest {
 };
 
 
+typedef struct server_conn {
+	/* hash keys and list for collision resolution */
+	__u32			addr;	/* IP address of the server */
+	__u16			port;	/* port number of the server */
+	struct list_head	list;	/* d-linked list head for hashing */
+
+	/* socket connected to a destination server */
+	struct socket		*sock;
+
+	/* timer for keepalive connections */
+	struct timer_list	keepalive_timer;
+	unsigned long		timeout;
+	unsigned int		nr_keepalives;
+} server_conn_t;
+
+
 /*
  *      TCPVS connection object
  */
@@ -342,8 +371,7 @@ struct tcp_vs_scheduler {
 	int (*update_service)(struct tcp_vs_service *svc);
 
 	/* select a server and connect to it */
-	struct socket * (*schedule)(struct tcp_vs_conn *conn,
-				    struct tcp_vs_service *svc);
+	int (*schedule)(struct tcp_vs_conn *conn, struct tcp_vs_service *svc);
 };
 
 
@@ -368,6 +396,8 @@ extern int tcp_vs_recvbuffer(struct socket *sock, char *buffer,
 extern int tcp_vs_exp_comp(struct tcp_vs_expression *exp, const char *pattern);
 extern int tcp_vs_exp_exec(struct tcp_vs_expression *exp, const char *string);
 extern void tcp_vs_exp_free(struct tcp_vs_expression *exp);
+extern int tcp_vs_xmit(struct socket *sock, const char *buffer,
+		       const size_t length, unsigned long flags);
 
 
 #ifndef strdup
