@@ -1,7 +1,7 @@
 /*
  *      tcpvsadm - TCP Virtual Server ADMinistration program
  *
- *      Version: $Id: tcpvsadm.c,v 1.8 2002/05/05 12:12:19 wensong Exp $
+ *      Version: $Id: tcpvsadm.c,v 1.9 2003/02/14 09:22:39 wensong Exp $
  *
  *      Authors: Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -628,7 +628,7 @@ static void usage_exit(const int status)
 		"  --add-service     -A        add virtual service with options\n"
 		"  --edit-service    -E        edit virtual service with options\n"
 		"  --delete-service  -D        delete virtual service\n"
-		"  --clear           -C        clear the whole table\n"
+		"  --flush           -F        flush the whole table\n"
 		"  --add-server      -a        add real server with options\n"
 		"  --edit-server     -e        edit real server with options\n"
 		"  --delete-server   -d        delete real server\n"
@@ -636,6 +636,8 @@ static void usage_exit(const int status)
 		"  --del-rule                  del rule into virtual service\n"
 		"  --list            -L|-l     list the table\n"
 		"  --load-configfile -f        load a config file\n"
+		"  --start                     start the virtual services\n"
+		"  --stop                      stop the virtual services\n"
 		"  --help            -h        display this help message\n\n"
 		);
 
